#!/usr/bin/env python
# *****************************************************************************
# Copyright (C) 2023 Thomas Touhey <thomas@touhey.fr>
#
# This software is governed by the CeCILL 2.1 license under French law and
# abiding by the rules of distribution of free software. You can use, modify
# and/or redistribute the software under the terms of the CeCILL 2.1 license as
# circulated by CEA, CNRS and INRIA at the following URL: https://cecill.info
#
# As a counterpart to the access to the source code and rights to copy, modify
# and redistribute granted by the license, users are provided only with a
# limited warranty and the software's author, the holder of the economic
# rights, and the successive licensors have only limited liability.
#
# In this respect, the user's attention is drawn to the risks associated with
# loading, using, modifying and/or developing or reproducing the software by
# the user in light of its specific status of free software, that may mean that
# it is complicated to manipulate, and that also therefore means that it is
# reserved for developers and experienced professionals having in-depth
# computer knowledge. Users are therefore encouraged to load and test the
# software's suitability as regards their requirements in conditions enabling
# the security of their systems and/or data to be ensured and, more generally,
# to use and operate it in the same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL 2.1 license and that you accept its terms.
# *****************************************************************************
"""Bind a callback state and read related events using TeaL.

This makes use of the TeaL Websocket Dispatcher, on a local TeaL stack run
using the Docker compose file present in the repository.
"""

from __future__ import annotations

from base64 import b64encode
from datetime import datetime, timedelta
from json import dumps, loads
from secrets import token_hex

import click
from requests import get
from websockets.sync.client import connect

__all__ = ['cli']

DISPATCHER_BASEURL: str = 'http://localhost:8081'
DISPATCHER_PASSWORD: str = 'abc'


@click.command()
@click.option('--final-redirect-url', required=True)
def cli(final_redirect_url: str) -> None:
    """Bind a callback state on TeaL, and read resulting events."""
    # Get the dispatcher metadata.
    response = get(
        DISPATCHER_BASEURL + '/.well-known/teapots-teal-metadata',
        timeout=10,
    )
    data = response.json()

    teal_redirect_url = data['callback_url']
    teal_error_url = data['errback_url']
    teal_websocket_url = data['websocket_url']

    # Build a random state.
    state = token_hex(8)

    # Connect to the websocket dispatcher.
    auth_header = 'Basic ' + b64encode(
        ('anonymous:' + DISPATCHER_PASSWORD).encode('ascii'),
    ).decode('ascii')

    websocket = connect(
        teal_websocket_url,
        additional_headers={'Authorization': auth_header},
    )

    # Bind the state.
    expires_at = datetime.utcnow() + timedelta(days=7)
    websocket.send(dumps({
        'create': {
            'type': 'callback',
            'state': state,
            'final_redirect_url': final_redirect_url,
            'with_fragment': False,
            'expires_at': expires_at.isoformat(),
        },
        'register_to': {
            'type': 'callback',
            'state': state,
        },
    }))

    print()
    print('The following state has been bound with the provided parameters:')
    print('')
    print('* Redirect URL:', teal_redirect_url)
    print('* Error URL:', teal_error_url)
    print('* State:', state)
    print()
    print('The received events will be displayed hereafter:')
    print()
    print('---')

    while True:
        # Receive callback messages.
        message = websocket.recv()
        data = loads(message)

        print()
        print('A callback has been received at ' + data['timestamp'] + '!')
        print(data['url'])


if __name__ == '__main__':
    cli()
